/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/claw/claw.hpp>
#include <ox/event/signal.hpp>
#include <ox/std/string.hpp>

#include <studio/itemmaker.hpp>
#include <studio/popup.hpp>

namespace studio {

class NewProject: public studio::Popup {
	public:
		enum class Stage {
			Closed,
			Opening,
			NewItemName,
		};

		// emits path parameter
		ox::Signal<ox::Error(ox::StringView)> finished;

	private:
		Stage m_stage = Stage::Closed;
		ox::String const m_projectDataDir;
		ox::String m_projectPath;
		ox::BString<255> m_projectName;
		ox::Vector<ox::UniquePtr<studio::ItemMaker>> m_types;
		bool m_open = false;

	public:
		NewProject(ox::String projectDatadir) noexcept;

		void open() noexcept override;

		void close() noexcept override;

		[[nodiscard]]
		bool isOpen() const noexcept override;

		void draw(turbine::Context &ctx) noexcept override;

	private:
		void drawNewProjectName(turbine::Context &ctx) noexcept;

		void drawFirstPageButtons() noexcept;

		void drawLastPageButtons(turbine::Context &ctx) noexcept;

		void finish() noexcept;

};

}
